import Plugin from '@jbrowse/core/Plugin';
import { isAbstractMenuManager } from '@jbrowse/core/util';
// icons
import LineStyleIcon from '@mui/icons-material/LineStyle';
// locals
import { BaseLinearDisplay, BaseLinearDisplayComponent, BlockMsg, baseLinearDisplayConfigSchema, } from './BaseLinearDisplay';
import LinearBareDisplayF, { configSchemaFactory as linearBareDisplayConfigSchemaFactory, } from './LinearBareDisplay';
import LinearGenomeViewF, { renderToSvg, RefNameAutocomplete, SearchBox, ZoomControls, LinearGenomeView, } from './LinearGenomeView';
import LinearBasicDisplayF, { configSchema as linearBasicDisplayConfigSchemaFactory, modelFactory as linearBasicDisplayModelFactory, } from './LinearBasicDisplay';
import FeatureTrackF from './FeatureTrack';
import BasicTrackF from './BasicTrack';
import LaunchLinearGenomeViewF from './LaunchLinearGenomeView';
export default class LinearGenomeViewPlugin extends Plugin {
    constructor() {
        super(...arguments);
        this.name = 'LinearGenomeViewPlugin';
        this.exports = {
            BaseLinearDisplayComponent,
            BaseLinearDisplay,
            baseLinearDisplayConfigSchema,
            SearchBox,
            ZoomControls,
            LinearGenomeView,
        };
    }
    install(pluginManager) {
        FeatureTrackF(pluginManager);
        BasicTrackF(pluginManager);
        LinearBasicDisplayF(pluginManager);
        LinearGenomeViewF(pluginManager);
        LinearBareDisplayF(pluginManager);
        LaunchLinearGenomeViewF(pluginManager);
    }
    configure(pluginManager) {
        if (isAbstractMenuManager(pluginManager.rootModel)) {
            pluginManager.rootModel.appendToSubMenu(['Add'], {
                label: 'Linear genome view',
                icon: LineStyleIcon,
                onClick: (session) => {
                    session.addView('LinearGenomeView', {});
                },
            });
        }
    }
}
export { baseLinearDisplayConfigSchema, linearBareDisplayConfigSchemaFactory, linearBasicDisplayConfigSchemaFactory, linearBasicDisplayModelFactory, renderToSvg, BaseLinearDisplayComponent, BaseLinearDisplay, RefNameAutocomplete, SearchBox, BlockMsg, };
//# sourceMappingURL=index.js.map